#!/usr/bin/python
#
# A script to compute the overlapping lengths of genomic sequences using the
# EMBOSS water program.
#
import os
import sys

# ----------------------------------------------------------------------------
class WaterOutput:
    """
    A class that contains a subset of the output from the water program.
    """
    overlap = None
    start1 = None
    start2 = None
    
    # ------------------------------------------------------------------------
    def __init__(self, file1, file2):
        """
        Initializes a new instance of the WaterOutput class.  The fields of
        the class are initialized from the output of the water program.
        
        Args:
            self: A pointer to the instance.
            file1: The path to the first sequence file.
            file2: The path to the second sequence file.
        """
        output = 'water-output.emboss'
        
        t = 'water {0} {1} -aformat markx10 -gapopen 100.0 -gapextend 10.0 -auto -outfile {2}'
        c = t.format(file1, file2, output)
        os.system(c)
        
        # Read the temporary output file for the result.
        with open(output) as f:
            for s in f:
                s = s.replace('\n', '')
                
                # Check for the overlap.
                token = '; sw_overlap: '
                if s.startswith(token):
                    self.overlap = int(s[len(token):])
                    continue
                
                # Ignore lines not indicating start index.
                token = '; al_start: ';
                if not s.startswith(token):
                    continue
                
                # Parse the index and store it into the first or second field.
                v = int(s[len(token):])
                if self.start1 is None:
                    self.start1 = v
                    continue
                
                if self.start2 is None:
                    self.start2 = v
                    continue
                
                # Do not allow three (or more) starting sequences.
                sys.stderr.write('Found three al_start from water output.')
                sys.exit(1)
        
        # Ensure all fields were initialized.
        if self.overlap is None:
            sys.stderr.write('Did not parse overlap.')
            sys.exit(1)
        
        if self.start2 is None:
            sys.stderr.write('Did not parse two start indices.')
            sys.exit(1)
        
        # Delete the temporary file and return the overlapping length.
        os.remove(output)

# ----------------------------------------------------------------------------
def isSymbol(c):
    """
    Returns True if the specified symbol is A, T, C, or G.
    
    Args:
        c: A character.
    
    Returns:
        True if the character is A, C, T, or G; otherwise, False.
    """
    return 'ACTG'.find(c) >= 0

# ----------------------------------------------------------------------------
def parseFasta():
    """
    Parses FASTA data from standard input and returns an array of strings for
    each sequence.
    
    Returns:
        An array of strings for each sequence.
    """
    lines = []

    # Loop over each line from standard input.
    for s in sys.stdin:
        # Ignore lines that do not start with A, T, C, or G.
        if len(s) == 0 or not isSymbol(s[0]):
            continue

        # Do not return the end-of-line character.
        s = s.replace('\n', '')
        
        # Check for any bad characters in the sequence.
        for c in s:
            if not isSymbol(s[0]):
                sys.stderr.write('Bad character: {0}'.format(c))
                sys.exit(1)

        lines.append(s)

    return lines

# ----------------------------------------------------------------------------
def processFasta(lines):
    """
    Processes sequences of FASTA data and computes the pair-wise overlapping
    length of each combination.  Results are written to standard output.
    
    Args:
        lines: The sequences of FASTA data (lines of ACTG).
    """
    seq1 = 'seq1.txt'
    seq2 = 'seq2.txt'

    # Loop over each line in the array.
    for i in range(len(lines)):
        writeFile(seq1, lines[i])
        
        # Loop over each remaining line in the array.
        for j in range(i + 1, len(lines)):
            writeFile(seq2, lines[j])
            
            # Run the water program and filter the output.
            o = WaterOutput(seq1, seq2)
            if o.overlap >= 20 and (o.start1 == 1 or o.start2 == 1):
                
                # Print the greater starting sequence first.
                if o.start1 > o.start2: t = '{0}\t{1}\t{2}'
                else: t = '{1}\t{0}\t{2}'
                print t.format(i + 1, j + 1, o.overlap)
            
            os.remove(seq2)
        
        os.remove(seq1)

# ----------------------------------------------------------------------------
def writeFile(path, text):
    """
    Writes text to a file.  If the file exists, it is replaced.  If the file
    does not exist, it is created.
    
    Args:
        path: The path to the file.
        text: The text to write to the file.
    """
    with open(path, 'w') as f:
        f.write(text)

# ----------------------------------------------------------------------------
#
# Read the FASTA sequences from standard input and write the pair-wise
# overlapping lengths to standard output.
#
processFasta(parseFasta())
