#!/usr/bin/python
#
# A script that generates a digraph using graphviz for the output of the
# analyze.py script.
#
import sys
from yapgvb import *

# ----------------------------------------------------------------------------
class Edge:
    """
    A class that contains information about one edge in the graph.
    """
    src = None
    dst = None
    len = None
    
    def __init__(self, line):
        """
        Initializes a new instance of the Edge class.  The fields of the class
        are initialized from one line of output from the analyze.py script.
        
        Args:
            self: A pointer to the instance.
            line: A line of output from the analyze.py script.
        """
        # Read three tokens, one for each class field.
        s = line.split('\t');
        if len(s) != 3:
            sys.stderr.write('Bad edge: {0}.'.format(s))
            sys.exit(1)
        
        # Parse the values as integers.
        self.src = int(s[0])
        self.dst = int(s[1])
        self.len = int(s[2])

# ----------------------------------------------------------------------------
def addEdge(nodes, graph, edge):
    """
    Adds an edge to a graphiviz graph.
    
    Args:
        nodes: The ordered list of graphviz nodes.
        graph: The graphviz graph.
        edge: Edge information from the analyze.py output.
    #
    """
    n1 = nodes[edge.src - 1]
    n2 = nodes[edge.dst - 1]
    e = graph.add_edge(n1, n2)
    e.label = str(edge.len)

# ----------------------------------------------------------------------------
def addNode(nodes, graph, index):
    """
    Adds a node to a graphiviz graph.
    
    Args:
        nodes: The ordered list of graphviz nodes.
        graph: The graphviz graph.
        index: The zero-bound index of the node.
    #
    """
    n = graph.add_node()
    n.fillcolor = 'aliceblue'
    n.label = str(index + 1)
    n.shape = 'rectangle'
    n.style = 'filled'
    nodes.append(n)

# ----------------------------------------------------------------------------
def countNodes(edges):
    """
    Returns the number of nodes in the graph.  This is determined by
    calculating the maximum node index in the output from the analyze.py
    script.
    
    Args:
        edges: The list of edges from the analyze.py output.
    
    Returns:
        The number of nodes in the graph.
    """
    count = 0
    for e in edges:
        i = max(e.src, e.dst)
        count = max(i, count)
    
    return count

# ----------------------------------------------------------------------------
def createGraph(edges, nodeCount):
    """
    Creates and returns a graph for the specified edges.
    
    Args:
        edges: The list of edges from the analyze.py output.
        nodeCount: The number of nodes in the graph.
    
    Returns:
        A graphviz graph for the analyze.py output.
    """
    graph = Digraph()
    
    # First create all the nodes in order.
    n = []
    for i in range(nodeCount):
        addNode(n, graph, i)
    
    # Next connect the nodes with edges.
    for e in edges:
        addEdge(n, graph, e)
    
    return graph

# ----------------------------------------------------------------------------
def graph():
    """
    Creates a graph based on input from the analyze.py script.  The input is
    read from standard input, and the output is written in SVG format to a
    file named 'output.svg'.
    """
    edges = readEdges()
    nodeCount = countNodes(edges)

    if nodeCount == 0:
        sys.stderr.write('No edges.')
        sys.exit(1)

    graph = createGraph(edges, nodeCount)
    graph.layout('dot')
    graph.render('output.svg', 'svg')
    
# ----------------------------------------------------------------------------
def readEdges():
    """
    Returns a list of edges read from standard input.  The format of the input
    matches the format of the output from the analyze.py program.
    
    Returns:
        A list of edges read from the analyze.py output.
    """
    edges = []
    for s in sys.stdin:
        s = s.replace('\n', '')
        edges.append(Edge(s))
    
    return edges;

# ----------------------------------------------------------------------------
#
# Create a graph based on input from the analyze.py script.
#
graph()
